//
//  ViewController.m
//  HelloWorld
//
//  Created by Phidgets on 2016-03-31.
//  Copyright © 2016 Phidgets. All rights reserved.
//

#import "ViewController.h"

@implementation ViewController

#pragma mark Event handling setup functions
static void gotAttach(PhidgetManagerHandle manager, void *ctx, PhidgetHandle phid){
    Phidget_retain(phid);
    [(__bridge id)ctx performSelectorOnMainThread:@selector(onAttachHandler:)
                                       withObject:[NSValue valueWithPointer:phid]
                                    waitUntilDone:NO];
}

static void gotDetach(PhidgetManagerHandle manager, void *ctx, PhidgetHandle phid){
    Phidget_retain(phid);
    [(__bridge id)ctx performSelectorOnMainThread:@selector(onDetachHandler:)
                                       withObject:[NSValue valueWithPointer:phid]
                                    waitUntilDone:NO];
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    PhidgetManagerHandle man;
    
    result = PhidgetManager_create(&man);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to create manager"];
    }
    result = PhidgetManager_setOnAttachHandler(man, gotAttach, (__bridge void *)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to set attach handler"];
    }
    result = PhidgetManager_setOnDetachHandler(man, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to set detach handler"];
    }
    result = PhidgetManager_open(man);
    if(result != EPHIDGET_OK){
        [self addError:@"Failed to open manager"];
    }
}

#pragma mark Event handling functions
-(void)onAttachHandler:(NSValue*)phid{
    PhidgetHandle p = (PhidgetHandle)[phid pointerValue];
    const char* deviceName;
    int32_t serialNumber, channel;
    
    Phidget_getDeviceName(p, &deviceName);
    Phidget_getDeviceSerialNumber(p, &serialNumber);
    Phidget_getChannel(p, &channel);
    [outputText setString:[[outputText string] stringByAppendingString:[NSString stringWithFormat:@"Device Name: %@, Serial Number: %d, Channel: %d\n", [NSString stringWithCString:deviceName encoding:NSASCIIStringEncoding],serialNumber,channel]]];
    [outputText scrollToEndOfDocument:self];
    Phidget_release(&p);
}

-(void)onDetachHandler:(NSValue*)phid{
    PhidgetHandle p = (PhidgetHandle)[phid pointerValue];
    const char* deviceName;
    int32_t serialNumber,channel;
    
    Phidget_getDeviceName(p, &deviceName);
    Phidget_getDeviceSerialNumber(p, &serialNumber);
    Phidget_getChannel(p, &channel);
    [outputText setString:[[outputText string] stringByAppendingString:[NSString stringWithFormat:@"Goodbye device %@, Serial Number: %d, Channel: %d\n", [NSString stringWithCString:deviceName encoding:NSASCIIStringEncoding],serialNumber,channel ]]];
    [outputText scrollToEndOfDocument:self];
    Phidget_release(&p);
}
#pragma mark library error handler
-(void)addError:(NSString *)errorString{
    [outputText setString:[[outputText string] stringByAppendingString: errorString]];
    [outputText scrollToEndOfDocument:self];
}

- (void)setRepresentedObject:(id)representedObject {
    [super setRepresentedObject:representedObject];
}

@end
